@@ -6,15 +6,13 @@
 #include <RcppArmadillo.h>
 #include <iostream>
 #include <vector>
-#include "basis.h"
-#include "covmatrix_funs_01.h"
+#include <boost/math/special_functions/bessel.hpp>
+#include <boost/math/special_functions/gamma.hpp>
 
 using namespace Rcpp;
 using namespace arma;
 //[[Rcpp::depends(RcppArmadillo)]]
-
-
-
+//[[Rcpp::depends(BH)]]
 
 
 //' Isotropic Matern covariance function, nonstationary variances
